@@ -1,5 +1,5 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
+	(c) 1998, 1999, 2000 Miodrag Vallat and others - see file AUTHORS for
 	complete list.
 
 	This library is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
   
 /*==============================================================================
 
-  $Id: drv_AF.c,v 1.25 1999/06/28 03:00:05 miod Exp $
+  $Id: drv_AF.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Driver for output on AF audio server.
 
@@ -48,6 +48,10 @@
 #include "config.h"
 #endif
 
+#include "mikmod_internals.h"
+
+#ifdef DRV_AF
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -55,7 +59,6 @@
 
 #include <AF/AFlib.h>
 
-#include <mikmod_internals.h>
 
 /* Global variables */
 
@@ -153,10 +156,7 @@ static BOOL AF_PlayStart(void)
 static void AF_Exit(void)
 {
 	VC_Exit();
-	if (audiobuffer) {
-		free(audiobuffer);
-		audiobuffer=NULL;
-	}
+	_mm_free(audiobuffer);
 	if (AFaud) {
 		AFCloseAudioConn(AFaud);
 		AFaud=NULL;
@@ -179,7 +179,7 @@ static void AF_Update(void)
 	}
 }
 
-MDRIVER drv_AF={
+MIKMODAPI MDRIVER drv_AF={
 	NULL,
 	"AF driver",
 	"AudioFile driver v1.3",
@@ -213,4 +213,10 @@ MDRIVER drv_AF={
 	VC_VoiceRealVolume
 };
 
+#else
+
+MISSING(drv_AF);
+
+#endif
+
 /* ex:set ts=4: */
