@@ -1,5 +1,5 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
+	(c) 1998, 1999, 2000 Miodrag Vallat and others - see file AUTHORS for
 	complete list.
 
 	This library is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
   
 /*==============================================================================
 
-  $Id: drv_aix.c,v 1.24 1999/06/28 03:00:05 miod Exp $
+  $Id: drv_aix.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Driver for output to AIX series audio device
 
@@ -36,6 +36,10 @@
 #include "config.h"
 #endif
 
+#include "mikmod_internals.h"
+
+#ifdef DRV_AIX
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -46,8 +50,6 @@
 #include <sys/audio.h>
 #include <sys/acpa.h>
 
-#include <mikmod_internals.h>
-
 #define BUFFERSIZE 15
 static	int buffersize=1<<BUFFERSIZE;
 static	int fd=-1;
@@ -69,6 +71,13 @@ static void AIX_CommandLine(CHAR *cmdline)
 
 static BOOL AIX_IsThere(void)
 {
+	int fd;
+
+	if((fd=open("/dev/acpa0/1",O_WRONLY))<0)
+		if((fd=open("/dev/paud0/1",O_WRONLY))<0)
+			if((fd=open("/dev/baud0/1",O_WRONLY))<0)
+				return 0;
+	close(fd);
 	return 1;
 }
 
@@ -142,10 +151,7 @@ static void AIX_Exit(void)
 		close(fd);
 		fd=-1;
 	}
-	if (audiobuffer) {
-		free(audiobuffer);
-		audiobuffer=NULL;
-	}
+	_mm_free(audiobuffer);
 }
 
 static void AIX_Update(void)
@@ -158,7 +164,7 @@ static void AIX_Pause(void)
 	write(fd,audiobuffer,VC_SilenceBytes(audiobuffer,buffersize));
 }
 
-MDRIVER drv_aix={
+MIKMODAPI MDRIVER drv_aix={
 	NULL,
 	"AIX Audio",
 	"AIX Audio driver v1.2",
@@ -192,4 +198,10 @@ MDRIVER drv_aix={
 	VC_VoiceRealVolume
 };
 
+#else
+
+MISSING(drv_aix);
+
+#endif
+
 /* ex:set ts=4: */
