@@ -1,5 +1,5 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
+	(c) 1998, 1999, 2000 Miodrag Vallat and others - see file AUTHORS for
 	complete list.
 
 	This library is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_alsa.c,v 1.21 1999/06/28 03:00:05 miod Exp $
+  $Id: drv_alsa.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Driver for Advanced Linux Sound Architecture (ALSA)
 
@@ -30,9 +30,17 @@
 #include "config.h"
 #endif
 
+#include "mikmod_internals.h"
+
+#ifdef DRV_ALSA
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
+
 #ifdef MIKMOD_DYNAMIC
 #include <dlfcn.h>
 #endif
@@ -40,26 +48,34 @@
 #include <string.h>
 
 #include <sys/asoundlib.h>
+#if defined(SND_LIB_VERSION) && (SND_LIB_VERSION >= 0x500)
+#undef DRV_ALSA
+#endif
 
-#include <mikmod_internals.h>
+#endif
+#ifdef DRV_ALSA
 
 #ifdef MIKMOD_DYNAMIC
 /* runtime link with libasound */
 static unsigned int(*alsa_cards_mask)(void);
-static int(*alsa_ctl_close)(void*);
-static int(*alsa_ctl_hw_info)(void*,struct snd_ctl_hw_info*);
-static int(*alsa_ctl_open)(void**,int);
-static int(*alsa_ctl_pcm_info)(void*,int,snd_pcm_info_t*);
-static int(*alsa_ctl_pcm_playback_info)(void*,int,snd_pcm_playback_info_t*);
-static int(*alsa_pcm_close)(void*);
-static int(*alsa_pcm_drain_playback)(void*);
-static int(*alsa_pcm_flush_playback)(void*);
-static int(*alsa_pcm_open)(void**,int,int,int);
-static int(*alsa_pcm_playback_format)(void*,snd_pcm_format_t*);
-static int(*alsa_pcm_playback_info)(void*,snd_pcm_playback_info_t*);
-static int(*alsa_pcm_playback_params)(void*,snd_pcm_playback_params_t*);
-static int(*alsa_pcm_playback_status)(void*,snd_pcm_playback_status_t*);
-static int(*alsa_pcm_write)(void*,const void*,size_t);
+static int(*alsa_ctl_close)(snd_ctl_t*);
+static int(*alsa_ctl_hw_info)(snd_ctl_t*,struct snd_ctl_hw_info*);
+static int(*alsa_ctl_open)(snd_ctl_t**,int);
+static int(*alsa_ctl_pcm_info)(snd_ctl_t*,int,snd_pcm_info_t*);
+#if defined(SND_LIB_VERSION) && (SND_LIB_VERSION >= 0x400)
+static int(*alsa_ctl_pcm_playback_info)(snd_ctl_t*,int,int,snd_pcm_playback_info_t*);
+#else
+static int(*alsa_ctl_pcm_playback_info)(snd_ctl_t*,int,snd_pcm_playback_info_t*);
+#endif
+static int(*alsa_pcm_close)(snd_pcm_t*);
+static int(*alsa_pcm_drain_playback)(snd_pcm_t*);
+static int(*alsa_pcm_flush_playback)(snd_pcm_t*);
+static int(*alsa_pcm_open)(snd_pcm_t**,int,int,int);
+static int(*alsa_pcm_playback_format)(snd_pcm_t*,snd_pcm_format_t*);
+static int(*alsa_pcm_playback_info)(snd_pcm_t*,snd_pcm_playback_info_t*);
+static int(*alsa_pcm_playback_params)(snd_pcm_t*,snd_pcm_playback_params_t*);
+static int(*alsa_pcm_playback_status)(snd_pcm_t*,snd_pcm_playback_status_t*);
+static int(*alsa_pcm_write)(snd_pcm_t*,const void*,size_t);
 static void* libasound=NULL;
 #ifndef HAVE_RTLD_GLOBAL
 #define RTLD_GLOBAL (0)
@@ -85,7 +101,7 @@ static void* libasound=NULL;
 
 #define DEFAULT_NUMFRAGS 4
 
-static	void* pcm_h=NULL;
+static	snd_pcm_t *pcm_h=NULL;
 static	int fragmentsize,numfrags=DEFAULT_NUMFRAGS;
 static	SBYTE *audiobuffer=NULL;
 static	int cardmin=0,cardmax=SND_CARDS;
@@ -212,7 +228,7 @@ static BOOL ALSA_Init_internal(void)
 	_mm_errno=MMERR_OPENING_AUDIO;
 	for (card=cardmin;card<cardmax;card++) {
 		struct snd_ctl_hw_info info;
-		void *ctl_h;
+		snd_ctl_t *ctl_h;
 		int dev,devmin,devmax;
 
 		/* no card here, onto the next */
@@ -249,8 +265,13 @@ static BOOL ALSA_Init_internal(void)
 				continue;
 
 			/* get playback information */
+#if defined(SND_LIB_VERSION) && (SND_LIB_VERSION >= 0x400)
+			if(alsa_ctl_pcm_playback_info(ctl_h,dev,0,&ctlinfo)<0)
+				continue;
+#else
 			if(alsa_ctl_pcm_playback_info(ctl_h,dev,&ctlinfo)<0)
 				continue;
+#endif
 
 	/*
 	   If control goes here, we have found a sound device able to play PCM data.
@@ -296,7 +317,7 @@ static BOOL ALSA_Init_internal(void)
 			bps=(pformat.rate*pformat.channels*(md_mode&DMODE_16BITS?2:1))>>1;
 			size=fragmentsize;while (size>bps) size>>=1;
 #ifdef MIKMOD_DEBUG
-			if (size<16) {
+			if (size < 16) {
 				fprintf(stderr,"\rweird hint result:%d from %d, bps=%d\n",size,fragmentsize,bps);
 				size=16;
 			}
@@ -349,10 +370,7 @@ static void ALSA_Exit_internal(void)
 		alsa_pcm_close(pcm_h);
 		pcm_h=NULL;
 	}
-	if (audiobuffer) {
-		free(audiobuffer);
-		audiobuffer=NULL;
-	}
+	_mm_free(audiobuffer);
 }
 
 static void ALSA_Exit(void)
@@ -365,18 +383,36 @@ static void ALSA_Exit(void)
 
 static void ALSA_Update(void)
 {
-#if 0
-	/* looks like a good idea, but causes *a lot* of clicks */
 	snd_pcm_playback_status_t status;
+	int total, count;
 
-	if(alsa_pcm_playback_status(pcm_h,&status)>=0)
-		/* Don't send data if ALSA is busy */
-		if(status.count)
-			alsa_pcm_write(pcm_h,audiobuffer,VC_WriteBytes(audiobuffer,
-			              status.count>fragmentsize?fragmentsize:status.count));
-#else
-	alsa_pcm_write(pcm_h,audiobuffer,VC_WriteBytes(audiobuffer,fragmentsize));
-#endif
+	if (alsa_pcm_playback_status(pcm_h, &status) >= 0) {
+		/* Update md_mixfreq if necessary */
+		if (md_mixfreq != status.rate)
+			md_mixfreq = status.rate;
+
+		/* Using status.count would cause clicks, as this is always less than
+		   the freespace  in the buffer - so compute how many bytes we can
+		   afford */
+		total = status.fragments * status.fragment_size - status.queue;
+		if (total < fragmentsize)
+			total = fragmentsize;
+	} else
+		total = fragmentsize;
+	
+	/* Don't send data if ALSA is too busy */
+	while (total) {
+		count = fragmentsize > total ? total : fragmentsize;
+		total -= count;
+		alsa_pcm_write(pcm_h,audiobuffer,VC_WriteBytes(audiobuffer,count));
+	}
+}
+
+static void ALSA_PlayStop(void)
+{
+	VC_PlayStop();
+	
+	alsa_pcm_flush_playback(pcm_h);
 }
 
 static BOOL ALSA_Reset(void)
@@ -385,10 +421,10 @@ static BOOL ALSA_Reset(void)
 	return ALSA_Init_internal();
 }
 
-MDRIVER drv_alsa={
+MIKMODAPI MDRIVER drv_alsa={
 	NULL,
 	"ALSA",
-	"Advanced Linux Sound Architecture (ALSA) driver v0.3.2",
+	"Advanced Linux Sound Architecture (ALSA) driver v0.4",
 	0,255,
 	"alsa",
 
@@ -403,7 +439,7 @@ MDRIVER drv_alsa={
 	ALSA_Reset,
 	VC_SetNumVoices,
 	VC_PlayStart,
-	VC_PlayStop,
+	ALSA_PlayStop,
 	ALSA_Update,
 	NULL,
 	VC_VoiceSetVolume,
@@ -419,4 +455,10 @@ MDRIVER drv_alsa={
 	VC_VoiceRealVolume
 };
 
+#else
+
+MISSING(drv_alsa);
+
+#endif
+
 /* ex:set ts=4: */
