@@ -1,5 +1,5 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
+	(c) 1998, 1999, 2000 Miodrag Vallat and others - see file AUTHORS for
 	complete list.
 
 	This library is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_esd.c,v 1.25 1999/06/28 03:00:05 miod Exp $
+  $Id: drv_esd.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Driver for the Enlightened sound daemon (EsounD)
 
@@ -37,6 +37,10 @@
 #include "config.h"
 #endif
 
+#include "mikmod_internals.h"
+
+#ifdef DRV_ESD
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -53,17 +57,11 @@
 
 #include <esd.h>
 
-#include <mikmod_internals.h>
-
 #ifdef MIKMOD_DYNAMIC
 /* runtime link with libesd */
 /* note that since we only need the network part of EsounD, we don't need to
    load libasound.so or libaudiofile.so as well */
-#ifdef HAVE_ESD_CLOSE
 static int(*esd_closestream)(int);
-#else
-#define esd_closestream esd_close
-#endif
 static int(*esd_playstream)(esd_format_t,int,char*,char*);
 static void* libesd=NULL;
 #ifndef HAVE_RTLD_GLOBAL
@@ -75,21 +73,17 @@ static void* libesd=NULL;
 #define esd_playstream esd_play_stream
 #endif
 
+#ifdef HAVE_SETENV
+#define SETENV setenv("ESD_NO_SPAWN","1",0)
+#else
+#define SETENV putenv("ESD_NO_SPAWN=1")
+#endif
+
 static	int sndfd=-1;
 static	esd_format_t format;
 static	SBYTE *audiobuffer=NULL;
 static	CHAR *espeaker=NULL;
 
-/* Backwards compatibility stuff */
-
-#ifndef HAVE_ESD_CLOSE
-#define esd_close(x) close(x)
-#endif
-
-#ifndef ESD_DEFAULT_RATE
-#define ESD_DEFAULT_RATE (44100)
-#endif
-
 #ifdef MIKMOD_DYNAMIC
 /* straight from audio.c in esd sources */
 esd_format_t esd_audio_format=ESD_BITS16|ESD_STEREO;
@@ -106,13 +100,11 @@ static BOOL ESD_Link(void)
 	if (libesd) return 0;
 
 	/* load libesd.so */
-	libesd=dlopen("libesd.so.0",RTLD_LAZY|RTLD_GLOBAL);
+	libesd=dlopen("libesd.so",RTLD_LAZY|RTLD_GLOBAL);
 	if (!libesd) return 1;
 
 	/* resolve function references */
-#ifdef HAVE_ESD_CLOSE
 	if (!(esd_closestream=dlsym(libesd,"esd_close"))) return 1;
-#endif
 	if (!(esd_playstream=dlsym(libesd,"esd_play_stream"))) return 1;
 
 	return 0;
@@ -170,7 +162,7 @@ static BOOL ESD_IsThere(void)
 #endif
 	/* Try to esablish a connection with default esd settings, but we don't
 	   want esdlib to spawn esd if esd is not running ! */
-	if (setenv("ESD_NO_SPAWN","1",0))
+	if (SETENV)
 		retval=0;
 	else {
 		if ((fd=esd_playstream(ESD_BITS16|ESD_STEREO|ESD_STREAM|ESD_PLAY,
@@ -191,9 +183,12 @@ static BOOL ESD_Init_internal(void)
 {
 	format=(md_mode&DMODE_16BITS?ESD_BITS16:ESD_BITS8)|
 	       (md_mode&DMODE_STEREO?ESD_STEREO:ESD_MONO)|ESD_STREAM|ESD_PLAY;
+	
+	if (md_mixfreq > ESD_DEFAULT_RATE)
+		md_mixfreq = ESD_DEFAULT_RATE;
 
 	/* make sure we can open an esd stream with our parameters */
-	if (!(setenv("ESD_NO_SPAWN","1",0))) {
+	if (!(SETENV)) {
 		if ((sndfd=esd_playstream(format,md_mixfreq,espeaker,"libmikmod"))<0) {
 			_mm_errno=MMERR_OPENING_AUDIO;
 			return 1;
@@ -223,10 +218,7 @@ static BOOL ESD_Init(void)
 static void ESD_Exit_internal(void)
 {
 	VC_Exit();
-	if (audiobuffer) {
-		free(audiobuffer);
-		audiobuffer=NULL;
-	}
+	_mm_free(audiobuffer);
 	if (sndfd>=0) {
 		esd_closestream(sndfd);
 		sndfd=-1;
@@ -261,7 +253,7 @@ static void ESD_Update_internal(int count)
 		if (time(NULL)-losttime>=5) {
 			losttime=time(NULL);
 			/* Attempt to reconnect every 5 seconds */
-			if (!(setenv("ESD_NO_SPAWN","1",0)))
+			if (!(SETENV))
 				if ((sndfd=esd_playstream(format,md_mixfreq,espeaker,"libmikmod"))>=0) {
 					VC_SilenceBytes(audiobuffer,ESD_BUF_SIZE);
 					esd_writebuf(sndfd,audiobuffer,ESD_BUF_SIZE);
@@ -283,7 +275,7 @@ static void ESD_Pause(void)
 static BOOL ESD_PlayStart(void)
 {
 	if (sndfd<0)
-		if (!(setenv("ESD_NO_SPAWN","1",0)))
+		if (!(SETENV))
 			if ((sndfd=esd_playstream(format,md_mixfreq,espeaker,"libmikmod"))<0) {
 				_mm_errno=MMERR_OPENING_AUDIO;
 				return 1;
@@ -320,11 +312,11 @@ static BOOL ESD_Reset(void)
 	return ESD_Init_internal();
 }
 
-MDRIVER drv_esd={
+MIKMODAPI MDRIVER drv_esd={
 	NULL,
 	"Enlightened sound daemon",
-	/* use the same version number as the EsounD release it works with */
-	"Enlightened sound daemon (EsounD) driver v0.2.12",
+	/* use the same version number as the EsounD release it works best with */
+	"Enlightened sound daemon (EsounD) driver v0.2.23",
 	0,255,
 	"esd",
 
@@ -355,4 +347,10 @@ MDRIVER drv_esd={
 	VC_VoiceRealVolume
 };
 
+#else
+
+MISSING(drv_esd);
+
+#endif
+
 /* ex:set ts=4: */
