@@ -1,6 +1,6 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
-	complete list.
+	(c) 1998, 1999, 2000, 2001, 2002 Miodrag Vallat and others - see file
+	AUTHORS for complete list.
 
 	This library is free software; you can redistribute it and/or modify
 	it under the terms of the GNU Library General Public License as
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_imf.c,v 1.10 1999/06/28 03:00:05 miod Exp $
+  $Id: load_imf.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Imago Orpheus (IMF) module loader
 
@@ -30,9 +30,21 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <stdio.h>
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
 #include <string.h>
 
-#include <mikmod_internals.h>
+#include "mikmod_internals.h"
+
+#ifdef SUNOS
+extern int fprintf(FILE *, const char *, ...);
+#endif
 
 /*========== Module structure */
 
@@ -135,11 +147,8 @@ void IMF_Cleanup(void)
 {
 	FreeLinear();
 
-	if(imfpat) free(imfpat);
-	if(mh) free(mh);
-
-	imfpat=NULL;
-	mh=NULL;
+	_mm_free(imfpat);
+	_mm_free(mh);
 }
 
 static BOOL IMF_ReadPattern(SLONG size,UWORD rows)
@@ -209,11 +218,10 @@ static void IMF_ProcessCmd(UBYTE eff,UBYTE inf)
 				UniEffect(UNI_S3MEFFECTD,0);
 				break;
 			case 0x05:	/* vibrato */
-				UniPTEffect(0x4,inf);
+				UniEffect(UNI_XMEFFECT4,inf);
 				break;
 			case 0x06:	/* vibrato + volslide */
-				UniPTEffect(0x4,inf);
-				UniEffect(UNI_S3MEFFECTD,0);
+				UniEffect(UNI_XMEFFECT6,inf);
 				break;
 			case 0x07:	/* fine vibrato */
 				UniEffect(UNI_ITEFFECTU,inf);
@@ -264,7 +272,7 @@ static void IMF_ProcessCmd(UBYTE eff,UBYTE inf)
 			case 0x14:	/* fine slide up */
 				if (inf) {
 					if (inf<0x40)
-						UniEffect(UNI_S3MEFFECTF,0xe0|(inf/4));
+						UniEffect(UNI_S3MEFFECTF,0xe0|(inf>>2));
 					else
 						UniEffect(UNI_S3MEFFECTF,0xf0|(inf>>4));
 				} else
@@ -273,7 +281,7 @@ static void IMF_ProcessCmd(UBYTE eff,UBYTE inf)
 			case 0x15:	/* fine slide down */
 				if (inf) {
 					if (inf<0x40)
-						UniEffect(UNI_S3MEFFECTE,0xe0|(inf/4));
+						UniEffect(UNI_S3MEFFECTE,0xe0|(inf>>2));
 					else
 						UniEffect(UNI_S3MEFFECTE,0xf0|(inf>>4));
 				} else
@@ -306,7 +314,7 @@ static void IMF_ProcessCmd(UBYTE eff,UBYTE inf)
 				UniPTEffect(0xd,(inf>>4)*10+(inf&0xf));
 				break;
 			case 0x1f:	/* set master volume */
-				if(inf<=64) UniEffect(UNI_XMEFFECTG,inf);
+				if(inf<=64) UniEffect(UNI_XMEFFECTG,inf<<1);
 				break;
 			case 0x20:	/* master volume slide */
 				UniEffect(UNI_XMEFFECTH,inf);
@@ -330,12 +338,11 @@ static void IMF_ProcessCmd(UBYTE eff,UBYTE inf)
 					case 0xf:	/* invert loop */
 						UniPTEffect(0xe,inf);
 						break;
-#ifdef MIKMOD_DEBUG
 					case 0xe:	/* ignore envelope */
-						fprintf(stderr,"\reffect 0x21[Ex] (ignore envelope)"
-						               " not implemented (inf=%2X)\n",inf);
+						UniEffect(UNI_ITEFFECTS0, 0x77);    /* vol */
+						UniEffect(UNI_ITEFFECTS0, 0x79);    /* pan */
+						UniEffect(UNI_ITEFFECTS0, 0x7b);    /* pit */
 						break;
-#endif
 				}
 				break;
 			/* 0x22 chorus (awe32) */
@@ -373,7 +380,7 @@ static UBYTE* IMF_ConvertTrack(IMFNOTE* tr,UWORD rows)
 BOOL IMF_Load(BOOL curious)
 {
 #define IMF_SMPINCR 64
-	int t,u,track=0;
+	int t,u,track=0,oldnumsmp;
 	IMFCHANNEL channels[32];
 	INSTRUMENT *d;
 	SAMPLE *q;
@@ -409,8 +416,9 @@ BOOL IMF_Load(BOOL curious)
 	of.initspeed=mh->initspeed;
 	of.inittempo=mh->inittempo;
 	of.initvolume=mh->mastervol<<1;
-	of.flags|=UF_INST;
-	if(mh->flags&1) of.flags|=UF_LINEAR;
+	of.flags |= UF_INST | UF_ARPMEM | UF_PANNING;
+	if(mh->flags&1) of.flags |= UF_LINEAR;
+	of.bpmlimit=32;
 
 	/* read channel information */
 	of.numchn=0;
@@ -486,7 +494,7 @@ BOOL IMF_Load(BOOL curious)
 	if(!AllocInstruments()) return 0;
 	d=of.instruments;
 
-	for(t=0;t<of.numins;t++) {
+	for(oldnumsmp=t=0;t<of.numins;t++) {
 		IMFINSTHEADER ih;
 
 		memset(d->samplenumber,0xff,INSTNOTES*sizeof(UWORD));
@@ -500,15 +508,27 @@ BOOL IMF_Load(BOOL curious)
 		_mm_read_I_UWORDS(ih.panenv,IMFENVCNT,modreader);
 		_mm_read_I_UWORDS(ih.pitenv,IMFENVCNT,modreader);
 
-#define IMF_FinishLoadingEnvelope(name)			\
-		ih.##name##pts=_mm_read_UBYTE(modreader);	\
-		ih.##name##sus=_mm_read_UBYTE(modreader);	\
-		ih.##name##beg=_mm_read_UBYTE(modreader);	\
-		ih.##name##end=_mm_read_UBYTE(modreader);	\
-		ih.##name##flg=_mm_read_UBYTE(modreader);	\
-		_mm_read_UBYTE(modreader);					\
+#if defined __STDC__ || defined _MSC_VER
+#define IMF_FinishLoadingEnvelope(name)					\
+		ih. name##pts=_mm_read_UBYTE(modreader);		\
+		ih. name##sus=_mm_read_UBYTE(modreader);		\
+		ih. name##beg=_mm_read_UBYTE(modreader);		\
+		ih. name##end=_mm_read_UBYTE(modreader);		\
+		ih. name##flg=_mm_read_UBYTE(modreader);		\
+		_mm_read_UBYTE(modreader);						\
+		_mm_read_UBYTE(modreader);						\
+		_mm_read_UBYTE(modreader)
+#else
+#define IMF_FinishLoadingEnvelope(name)				\
+		ih. name/**/pts=_mm_read_UBYTE(modreader);	\
+		ih. name/**/sus=_mm_read_UBYTE(modreader);	\
+		ih. name/**/beg=_mm_read_UBYTE(modreader);	\
+		ih. name/**/end=_mm_read_UBYTE(modreader);	\
+		ih. name/**/flg=_mm_read_UBYTE(modreader);	\
 		_mm_read_UBYTE(modreader);					\
 		_mm_read_UBYTE(modreader);					\
+		_mm_read_UBYTE(modreader)
+#endif
 
 		IMF_FinishLoadingEnvelope(vol);
 		IMF_FinishLoadingEnvelope(pan);
@@ -518,12 +538,17 @@ BOOL IMF_Load(BOOL curious)
 		ih.numsmp =_mm_read_I_UWORD(modreader);
 
 		_mm_read_UBYTES(id,4,modreader);
-		if(memcmp(id,"II10",4)) {
+		/* Looks like Imago Orpheus forgets the signature for empty
+		   instruments following a multi-sample instrument... */
+		if(memcmp(id,"II10",4) && 
+		   (oldnumsmp && memcmp(id,"\x0\x0\x0\x0",4))) {
 			if(nextwav) free(nextwav);
 			if(wh) free(wh);
 			_mm_errno=MMERR_LOADING_SAMPLEINFO;
 			return 0;
 		}
+		oldnumsmp=ih.numsmp;
+
 		if((ih.numsmp>16)||(ih.volpts>IMFENVCNT/2)||(ih.panpts>IMFENVCNT/2)||
 		   (ih.pitpts>IMFENVCNT/2)||(_mm_eof(modreader))) {
 			if(nextwav) free(nextwav);
@@ -536,27 +561,49 @@ BOOL IMF_Load(BOOL curious)
 			d->samplenumber[u]=ih.what[u]>ih.numsmp?0xffff:ih.what[u]+of.numsmp;
 		d->volfade=ih.volfade;
 
-#define IMF_ProcessEnvelope(name) \
-		memcpy(d->##name##env,ih.##name##env,IMFENVCNT);		\
-		if(ih.##name##flg&1) d->##name##flg|=EF_ON;				\
-		if(ih.##name##flg&2) d->##name##flg|=EF_SUSTAIN;		\
-		if(ih.##name##flg&4) d->##name##flg|=EF_LOOP;			\
-		d->##name##susbeg=d->##name##susend=ih.##name##sus;		\
-		d->##name##beg=ih.##name##beg;							\
-		d->##name##end=ih.##name##end;							\
-		d->##name##pts=ih.##name##pts;							\
-																\
-		if((d->##name##flg&EF_ON)&&(d->##name##pts<2))			\
-			d->##name##flg&=~EF_ON;								\
+#if defined __STDC__ || defined _MSC_VER
+#define IMF_ProcessEnvelope(name) 									\
+		for (u = 0; u < (IMFENVCNT >> 1); u++) {					\
+			d-> name##env[u].pos = ih. name##env[u << 1];			\
+			d-> name##env[u].val = ih. name##env[(u << 1)+ 1];		\
+		}															\
+		if (ih. name##flg&1) d-> name##flg|=EF_ON;					\
+		if (ih. name##flg&2) d-> name##flg|=EF_SUSTAIN;				\
+		if (ih. name##flg&4) d-> name##flg|=EF_LOOP;				\
+		d-> name##susbeg=d-> name##susend=ih. name##sus;			\
+		d-> name##beg=ih. name##beg;								\
+		d-> name##end=ih. name##end;								\
+		d-> name##pts=ih. name##pts;								\
+																	\
+		if ((d-> name##flg&EF_ON)&&(d-> name##pts<2))				\
+			d-> name##flg&=~EF_ON
+#else
+#define IMF_ProcessEnvelope(name) 									\
+		for (u = 0; u < (IMFENVCNT >> 1); u++) {					\
+			d-> name/**/env[u].pos = ih. name/**/env[u << 1];		\
+			d-> name/**/env[u].val = ih. name/**/env[(u << 1)+ 1];	\
+		}															\
+		if (ih. name/**/flg&1) d-> name/**/flg|=EF_ON;				\
+		if (ih. name/**/flg&2) d-> name/**/flg|=EF_SUSTAIN;			\
+		if (ih. name/**/flg&4) d-> name/**/flg|=EF_LOOP;			\
+		d-> name/**/susbeg=d-> name/**/susend=ih. name/**/sus;		\
+		d-> name/**/beg=ih. name/**/beg;							\
+		d-> name/**/end=ih. name/**/end;							\
+		d-> name/**/pts=ih. name/**/pts;							\
+																	\
+		if ((d-> name/**/flg&EF_ON)&&(d-> name/**/pts<2))			\
+			d-> name/**/flg&=~EF_ON
+#endif
 
 		IMF_ProcessEnvelope(vol);
 		IMF_ProcessEnvelope(pan);
 		IMF_ProcessEnvelope(pit);
+#undef IMF_ProcessEnvelope
 
 		if(ih.pitflg&1) {
 			d->pitflg&=~EF_ON;
 #ifdef MIKMOD_DEBUG
-			fputs("\rFilter envelopes not supported yet\n",stderr);
+			fprintf(stderr, "\rFilter envelopes not supported yet\n");
 #endif
 		}
 
@@ -677,7 +724,7 @@ CHAR *IMF_LoadTitle(void)
 
 /*========== Loader information */
 
-MLOADER load_imf={
+MIKMODAPI MLOADER load_imf={
 	NULL,
 	"IMF",
 	"IMF (Imago Orpheus)",
