@@ -1,6 +1,6 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
-	complete list.
+	(c) 1998, 1999, 2000, 2001, 2002 Miodrag Vallat and others - see file
+	AUTHORS for complete list.
 
 	This library is free software; you can redistribute it and/or modify
 	it under the terms of the GNU Library General Public License as
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_ult.c,v 1.26 1999/06/28 03:00:05 miod Exp $
+  $Id: load_ult.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Ultratracker (ULT) module loader
 
@@ -30,9 +30,21 @@
 #include "config.h"
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <stdio.h>
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
 #include <string.h>
 
-#include <mikmod_internals.h>
+#include "mikmod_internals.h"
+
+#ifdef SUNOS
+extern int fprintf(FILE *, const char *, ...);
+#endif
 
 /*========== Module structure */
 
@@ -138,7 +150,7 @@ BOOL ULT_Load(BOOL curious)
 
 	/* read songtext */
 	if ((mh.id[14]>'1')&&(mh.reserved))
-		if(!ReadLinedComment(mh.reserved,32)) return 0;
+		if(!ReadLinedComment(mh.reserved * 32, 32)) return 0;
 
 	nos=_mm_read_UBYTE(modreader);
 	if(_mm_eof(modreader)) {
@@ -195,7 +207,10 @@ BOOL ULT_Load(BOOL curious)
 	for(t=0;t<256;t++)
 		of.positions[t]=_mm_read_UBYTE(modreader);
 	for(t=0;t<256;t++)
-		if(of.positions[t]==255) break;
+		if(of.positions[t]==255) {
+			of.positions[t]=LAST_PATTERN;
+			break;
+		}
 	of.numpos=t;
 
 	noc=_mm_read_UBYTE(modreader);
@@ -211,8 +226,10 @@ BOOL ULT_Load(BOOL curious)
 			of.patterns[(t*of.numchn)+u]=tracks++;
 
 	/* read pan position table for v1.5 and higher */
-	if(mh.id[14]>='3')
+	if(mh.id[14]>='3') {
 		for(t=0;t<of.numchn;t++) of.panning[t]=_mm_read_UBYTE(modreader)<<4;
+		of.flags |= UF_PANNING;
+	}
 
 	for(t=0;t<of.numtrk;t++) {
 		int rep,row=0;
@@ -248,6 +265,7 @@ BOOL ULT_Load(BOOL curious)
 						break;
 					case 0xb: /* panning */
 						UniPTEffect(8,ev.dat2*0xf);
+						of.flags |= UF_PANNING;
 						break;
 					case 0xc: /* volume */
 						UniPTEffect(eff,ev.dat2>>2);
@@ -271,6 +289,7 @@ BOOL ULT_Load(BOOL curious)
 						break;
 					case 0xb: /* panning */
 						UniPTEffect(8,ev.dat1*0xf);
+						of.flags |= UF_PANNING;
 						break;
 					case 0xc: /* volume */
 						UniPTEffect(eff,ev.dat1>>2);
@@ -301,7 +320,7 @@ CHAR *ULT_LoadTitle(void)
 
 /*========== Loader information */
 
-MLOADER load_ult={
+MIKMODAPI MLOADER load_ult={
 	NULL,
 	"ULT",
 	"ULT (UltraTracker)",
