@@ -1,6 +1,6 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
-	complete list.
+	(c) 1998, 1999, 2000, 2001, 2002 Miodrag Vallat and others - see file
+	AUTHORS for complete list.
 
 	This library is free software; you can redistribute it and/or modify
 	it under the terms of the GNU Library General Public License as
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mloader.c,v 1.31 1999/06/28 03:00:05 miod Exp $
+  $Id: mloader.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   These routines are used to access the available module loaders
 
@@ -30,10 +30,21 @@
 #include "config.h"
 #endif
 
-#include <mikmod_internals.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
 #include <string.h>
 
+#include "mikmod_internals.h"
+
+#ifdef SUNOS
+extern int fprintf(FILE *, const char *, ...);
+#endif
+
 		MREADER *modreader;
 		MODULE of;
 
@@ -44,24 +55,24 @@ UWORD finetune[16]={
 	7895,7941,7985,8046,8107,8169,8232,8280
 };
 
-CHAR* MikMod_InfoLoader(void)
+MIKMODAPI CHAR* MikMod_InfoLoader(void)
 {
 	int len=0;
 	MLOADER *l;
 	CHAR *list=NULL;
 
-	MUTEX_LOCK
-		/* compute size of buffer */
-		for(l=firstloader;l;l=l->next) len+=1+(l->next?1:0)+strlen(l->version);
+	MUTEX_LOCK(lists);
+	/* compute size of buffer */
+	for(l=firstloader;l;l=l->next) len+=1+(l->next?1:0)+strlen(l->version);
 
-		if(len)
-			if((list=_mm_malloc(len*sizeof(CHAR)))) {
-				list[0]=0;
-				/* list all registered module loders */
-				for(l=firstloader;l;l=l->next)
-					sprintf(list,(l->next)?"%s%s\n":"%s%s",list,l->version);
-			}
-	MUTEX_UNLOCK
+	if(len)
+		if((list=_mm_malloc(len*sizeof(CHAR)))) {
+			list[0]=0;
+			/* list all registered module loders */
+			for(l=firstloader;l;l=l->next)
+				sprintf(list,(l->next)?"%s%s\n":"%s%s",list,l->version);
+		}
+	MUTEX_UNLOCK(lists);
 	return list;
 }
 
@@ -76,23 +87,30 @@ void _mm_registerloader(MLOADER* ldr)
 		firstloader=ldr; 
 }
 
-void MikMod_RegisterLoader(struct MLOADER* ldr)
+MIKMODAPI void MikMod_RegisterLoader(struct MLOADER* ldr)
 {
 	/* if we try to register an invalid loader, or an already registered loader,
 	   ignore this attempt */
 	if ((!ldr)||(ldr->next))
 		return;
 
-	MUTEX_LOCK
-		_mm_registerloader(ldr);
-	MUTEX_UNLOCK
+	MUTEX_LOCK(lists);
+	_mm_registerloader(ldr);
+	MUTEX_UNLOCK(lists);
 }
 
 BOOL ReadComment(UWORD len)
 {
 	if(len) {
+		int i;
+
 		if(!(of.comment=(CHAR*)_mm_malloc(len+1))) return 0;
 		_mm_read_UBYTES(of.comment,len,modreader);
+		
+		/* translate IT linefeeds */
+		for(i=0;i<len;i++)
+			if(of.comment[i]=='\r') of.comment[i]='\n';
+
 		of.comment[len]=0;	/* just in case */
 	}
 	if(!of.comment[0]) {
@@ -102,18 +120,20 @@ BOOL ReadComment(UWORD len)
 	return 1;
 }
 
-BOOL ReadLinedComment(UWORD lines,UWORD linelen)
+BOOL ReadLinedComment(UWORD len,UWORD linelen)
 {
 	CHAR *tempcomment,*line,*storage;
-	UWORD total=0,t,len=lines*linelen;
+	UWORD total=0,t,lines;
 	int i;
 
-	if (lines) {
+	lines = (len + linelen - 1) / linelen;
+	if (len) {
 		if(!(tempcomment=(CHAR*)_mm_malloc(len+1))) return 0;
 		if(!(storage=(CHAR*)_mm_malloc(linelen+1))) {
 			free(tempcomment);
 			return 0;
 		}
+		memset(tempcomment, ' ', len);
 		_mm_read_UBYTES(tempcomment,len,modreader);
 
 		/* compute message length */
@@ -322,11 +342,11 @@ void Player_Free_internal(MODULE *mf)
 	}
 }
 
-void Player_Free(MODULE *mf)
+MIKMODAPI void Player_Free(MODULE *mf)
 {
-	MUTEX_LOCK
-		Player_Free_internal(mf);
-	MUTEX_UNLOCK
+	MUTEX_LOCK(vars);
+	Player_Free_internal(mf);
+	MUTEX_UNLOCK(vars);
 }
 
 static CHAR* Player_LoadTitle_internal(MREADER *reader)
@@ -353,7 +373,21 @@ static CHAR* Player_LoadTitle_internal(MREADER *reader)
 	return l->LoadTitle();
 }
 
-CHAR* Player_LoadTitle(CHAR* filename)
+MIKMODAPI CHAR* Player_LoadTitleFP(FILE *fp)
+{
+	CHAR* result=NULL;
+	MREADER* reader;
+
+	if(fp && (reader=_mm_new_file_reader(fp))) {
+		MUTEX_LOCK(lists);
+		result=Player_LoadTitle_internal(reader);
+		MUTEX_UNLOCK(lists);
+		_mm_delete_file_reader(reader);
+	}
+	return result;
+}
+
+MIKMODAPI CHAR* Player_LoadTitle(CHAR* filename)
 {
 	CHAR* result=NULL;
 	FILE* fp;
@@ -361,12 +395,12 @@ CHAR* Player_LoadTitle(CHAR* filename)
 
 	if((fp=_mm_fopen(filename,"rb"))) {
 		if((reader=_mm_new_file_reader(fp))) {
-			MUTEX_LOCK
-				result=Player_LoadTitle_internal(reader);
-			MUTEX_UNLOCK
+			MUTEX_LOCK(lists);
+			result=Player_LoadTitle_internal(reader);
+			MUTEX_UNLOCK(lists);
 			_mm_delete_file_reader(reader);
 		}
-		fclose(fp);
+		_mm_fclose(fp);
 	}
 	return result;
 }
@@ -404,23 +438,27 @@ MODULE* Player_LoadGeneric_internal(MREADER *reader,int maxchan,BOOL curious)
 		return NULL;
 	}
 
-	/* load the song using the song's loader variable */
+	/* init the module structure with vanilla settings */
 	memset(&of,0,sizeof(MODULE));
+	of.bpmlimit = 33;
 	of.initvolume = 128;
-
-	/* init panning array */
-	for(t=0; t<64; t++) of.panning[t] = ((t+1)&2) ? 255 : 0;
-	for(t=0; t<64; t++) of.chanvol[t] = 64;
+	for (t = 0; t < UF_MAXCHAN; t++) of.chanvol[t] = 64;
+	for (t = 0; t < UF_MAXCHAN; t++)
+		of.panning[t] = ((t + 1) & 2) ? PAN_RIGHT : PAN_LEFT;
 
 	/* init module loader and load the header / patterns */
-	if(l->Init()) {
+	if (!l->Init || l->Init()) {
 		_mm_rewind(modreader);
 		ok = l->Load(curious);
+		/* propagate inflags=flags for in-module samples */
+		for (t = 0; t < of.numsmp; t++)
+			if (of.samples[t].inflags == 0)
+				of.samples[t].inflags = of.samples[t].flags;
 	} else
 		ok = 0;
 
 	/* free loader and unitrk allocations */
-	l->Cleanup();
+	if (l->Cleanup) l->Cleanup();
 	UniCleanup();
 
 	if(!ok) {
@@ -443,10 +481,15 @@ MODULE* Player_LoadGeneric_internal(MREADER *reader,int maxchan,BOOL curious)
 		if(_mm_errorhandler) _mm_errorhandler();
 		return NULL;
 	}
+	
+	/* If the module doesn't have any specific panning, create a
+	   MOD-like panning, with the channels half-separated. */
+	if (!(of.flags & UF_PANNING))
+		for (t = 0; t < of.numchn; t++)
+			of.panning[t] = ((t + 1) & 2) ? PAN_HALFRIGHT : PAN_HALFLEFT;
 
 	/* Copy the static MODULE contents into the dynamic MODULE struct. */
 	memcpy(mf,&of,sizeof(MODULE));
-	_mm_iobase_revert();
 
 	if(maxchan>0) {
 		if(!(mf->flags&UF_NNA)&&(mf->numchn<maxchan))
@@ -458,35 +501,41 @@ MODULE* Player_LoadGeneric_internal(MREADER *reader,int maxchan,BOOL curious)
 		if(maxchan<mf->numchn) mf->flags |= UF_NNA;
 
 		if(MikMod_SetNumVoices_internal(maxchan,-1)) {
+			_mm_iobase_revert();
 			Player_Free(mf);
 			return NULL;
 		}
 	}
 	if(SL_LoadSamples()) {
+		_mm_iobase_revert();
 		Player_Free_internal(mf);
 		return NULL;
 	}
 	if(Player_Init(mf)) {
+		_mm_iobase_revert();
 		Player_Free_internal(mf);
 		mf=NULL;
 	}
+	_mm_iobase_revert();
 	return mf;
 }
 
-MODULE* Player_LoadGeneric(MREADER *reader,int maxchan,BOOL curious)
+MIKMODAPI MODULE* Player_LoadGeneric(MREADER *reader,int maxchan,BOOL curious)
 {
 	MODULE* result;
 
-	MUTEX_LOCK
+	MUTEX_LOCK(vars);
+	MUTEX_LOCK(lists);
 		result=Player_LoadGeneric_internal(reader,maxchan,curious);
-	MUTEX_UNLOCK
+	MUTEX_UNLOCK(lists);
+	MUTEX_UNLOCK(vars);
 
 	return result;
 }
 
 /* Loads a module given a file pointer.
    File is loaded from the current file seek position. */
-MODULE* Player_LoadFP(FILE* fp,int maxchan,BOOL curious)
+MIKMODAPI MODULE* Player_LoadFP(FILE* fp,int maxchan,BOOL curious)
 {
 	MODULE* result=NULL;
 	struct MREADER* reader=_mm_new_file_reader (fp);
@@ -500,14 +549,14 @@ MODULE* Player_LoadFP(FILE* fp,int maxchan,BOOL curious)
 
 /* Open a module via its filename.  The loader will initialize the specified
    song-player 'player'. */
-MODULE* Player_Load(CHAR* filename,int maxchan,BOOL curious)
+MIKMODAPI MODULE* Player_Load(CHAR* filename,int maxchan,BOOL curious)
 {
 	FILE *fp;
 	MODULE *mf=NULL;
 
 	if((fp=_mm_fopen(filename,"rb"))) {
 		mf=Player_LoadFP(fp,maxchan,curious);
-		fclose(fp);
+		_mm_fclose(fp);
 	}
 	return mf;
 }
