@@ -1,6 +1,6 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
-	complete list.
+	(c) 1998, 1999, 2000, 2001, 2002 Miodrag Vallat and others - see file
+	AUTHORS for complete list.
 
 	This library is free software; you can redistribute it and/or modify
 	it under the terms of the GNU Library General Public License as
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: virtch.c,v 1.31 1999/06/28 03:00:06 miod Exp $
+  $Id: virtch.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
 
   Sample mixing routines, using a 32 bits mixing buffer.
 
@@ -38,11 +38,14 @@
 #include "config.h"
 #endif
 
-#include <mikmod_internals.h>
-
 #include <stddef.h>
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
 #include <string.h>
 
+#include "mikmod_internals.h"
+
 /*
    Constant definitions
    ====================
@@ -130,12 +133,13 @@ static	SLONG *RVbufR1=NULL,*RVbufR2=NULL,*RVbufR3=NULL,*RVbufR4=NULL,
 static SLONG Mix32MonoNormal(SWORD* srce,SLONG* dest,SLONG index,SLONG increment,SLONG todo)
 {
 	SWORD sample;
+	SLONG lvolsel = vnf->lvolsel;
 
 	while(todo--) {
 		sample = srce[index >> FRACBITS];
 		index += increment;
 
-		*dest++ += vnf->lvolsel * sample;
+		*dest++ += lvolsel * sample;
 	}
 	return index;
 }
@@ -143,13 +147,15 @@ static SLONG Mix32MonoNormal(SWORD* srce,SLONG* dest,SLONG index,SLONG increment
 static SLONG Mix32StereoNormal(SWORD* srce,SLONG* dest,SLONG index,SLONG increment,SLONG todo)
 {
 	SWORD sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
 
 	while(todo--) {
 		sample=srce[index >> FRACBITS];
 		index += increment;
 
-		*dest++ += vnf->lvolsel * sample;
-		*dest++ += vnf->rvolsel * sample;
+		*dest++ += lvolsel * sample;
+		*dest++ += rvolsel * sample;
 	}
 	return index;
 }
@@ -157,17 +163,24 @@ static SLONG Mix32StereoNormal(SWORD* srce,SLONG* dest,SLONG index,SLONG increme
 static SLONG Mix32SurroundNormal(SWORD* srce,SLONG* dest,SLONG index,SLONG increment,SLONG todo)
 {
 	SWORD sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
 
-	while(todo--) {
-		sample = srce[index >> FRACBITS];
-		index += increment;
+	if (lvolsel>=rvolsel) {
+		while(todo--) {
+			sample = srce[index >> FRACBITS];
+			index += increment;
 
-		if(vnf->lvolsel>=vnf->rvolsel) {
-			*dest++ += vnf->lvolsel*sample;
-			*dest++ -= vnf->lvolsel*sample;
-		} else {
-			*dest++ -= vnf->rvolsel*sample;
-			*dest++ += vnf->rvolsel*sample;
+			*dest++ += lvolsel*sample;
+			*dest++ -= lvolsel*sample;
+		}
+	} else {
+		while(todo--) {
+			sample = srce[index >> FRACBITS];
+			index += increment;
+
+			*dest++ -= rvolsel*sample;
+			*dest++ += rvolsel*sample;
 		}
 	}
 	return index;
@@ -176,6 +189,26 @@ static SLONG Mix32SurroundNormal(SWORD* srce,SLONG* dest,SLONG index,SLONG incre
 static SLONG Mix32MonoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increment,SLONG todo)
 {
 	SLONG sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rampvol = vnf->rampvol;
+
+	if (rampvol) {
+		SLONG oldlvol = vnf->oldlvol - lvolsel;
+		while(todo--) {
+			sample=(SLONG)srce[index>>FRACBITS]+
+			       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+			        *(index&FRACMASK)>>FRACBITS);
+			index += increment;
+
+			*dest++ += ((lvolsel << CLICK_SHIFT) + oldlvol * rampvol)
+			           * sample >> CLICK_SHIFT;
+			if (!--rampvol)
+				break;
+		}
+		vnf->rampvol = rampvol;
+		if (todo < 0)
+			return index;
+	}
 
 	while(todo--) {
 		sample=(SLONG)srce[index>>FRACBITS]+
@@ -183,13 +216,7 @@ static SLONG Mix32MonoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increment
 		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
-		if(vnf->rampvol) {
-			*dest++ += (((SLONG)vnf->oldlvol<<CLICK_SHIFT) +
-			            (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
-			           )*sample>>CLICK_SHIFT;
-			vnf->rampvol--;
-		} else
-			*dest++ += vnf->lvolsel * sample;
+		*dest++ += lvolsel * sample;
 	}
 	return index;
 }
@@ -197,6 +224,30 @@ static SLONG Mix32MonoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increment
 static SLONG Mix32StereoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increment,SLONG todo)
 {
 	SLONG sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
+	SLONG rampvol = vnf->rampvol;
+
+	if (rampvol) {
+		SLONG oldlvol = vnf->oldlvol - lvolsel;
+		SLONG oldrvol = vnf->oldrvol - rvolsel;
+		while(todo--) {
+			sample=(SLONG)srce[index>>FRACBITS]+
+			       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+			        *(index&FRACMASK)>>FRACBITS);
+			index += increment;
+
+			*dest++ += ((lvolsel << CLICK_SHIFT) + oldlvol * rampvol)
+			           * sample >> CLICK_SHIFT;
+			*dest++ += ((rvolsel << CLICK_SHIFT) + oldrvol * rampvol)
+					   * sample >> CLICK_SHIFT;
+			if (!--rampvol)
+				break;
+		}
+		vnf->rampvol = rampvol;
+		if (todo < 0)
+			return index;
+	}
 
 	while(todo--) {
 		sample=(SLONG)srce[index>>FRACBITS]+
@@ -204,18 +255,8 @@ static SLONG Mix32StereoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increme
 		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
-		if(vnf->rampvol) {
-			*dest++ += (((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
-			            (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
-			           )*sample>>CLICK_SHIFT;
-			*dest++ += (((SLONG)vnf->oldrvol<<CLICK_SHIFT)+
-			            (SLONG)(vnf->oldrvol-vnf->rvolsel)*vnf->rampvol
-			           )*sample>>CLICK_SHIFT;
-			vnf->rampvol--;
-		} else {
-			*dest++ += vnf->lvolsel * sample;
-			*dest++ += vnf->rvolsel * sample;
-		}
+		*dest++ += lvolsel * sample;
+		*dest++ += rvolsel * sample;
 	}
 	return index;
 }
@@ -223,32 +264,48 @@ static SLONG Mix32StereoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increme
 static SLONG Mix32SurroundInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increment,SLONG todo)
 {
 	SLONG sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
+	SLONG rampvol = vnf->rampvol;
+	SLONG oldvol, vol;
+
+	if (lvolsel >= rvolsel) {
+		vol = lvolsel;
+		oldvol = vnf->oldlvol;
+	} else {
+		vol = rvolsel;
+		oldvol = vnf->oldrvol;
+	}
+
+	if (rampvol) {
+		oldvol -= vol;
+		while(todo--) {
+			sample=(SLONG)srce[index>>FRACBITS]+
+			       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+			        *(index&FRACMASK)>>FRACBITS);
+			index += increment;
+
+			sample=((vol << CLICK_SHIFT) + oldvol * rampvol)
+				   * sample >> CLICK_SHIFT;
+			*dest++ += sample;
+			*dest++ -= sample;
+
+			if (!--rampvol)
+				break;
+		}
+		vnf->rampvol = rampvol;
+		if (todo < 0)
+			return index;
+	}
 
 	while(todo--) {
-		int oldvol,vol;
-
 		sample=(SLONG)srce[index>>FRACBITS]+
 		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
 		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
-		if(vnf->lvolsel>=vnf->rvolsel) {
-			vol=vnf->lvolsel;oldvol=vnf->oldlvol;
-		} else {
-			vol=vnf->rvolsel;oldvol=vnf->oldrvol;
-		}
-
-		if(vnf->rampvol) {
-			sample=(((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
-			        (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
-			       )*sample>>CLICK_SHIFT;
-			*dest++ += sample;
-			*dest++ -= sample;
-			vnf->rampvol--;
-		} else {
-			*dest++ += vol*sample;
-			*dest++ -= vol*sample;
-		}
+		*dest++ += vol*sample;
+		*dest++ -= vol*sample;
 	}
 	return index;
 }
@@ -259,12 +316,13 @@ static SLONG Mix32SurroundInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG incre
 static SLONGLONG MixMonoNormal(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG increment,SLONG todo)
 {
 	SWORD sample;
+	SLONG lvolsel = vnf->lvolsel;
 
 	while(todo--) {
 		sample = srce[index >> FRACBITS];
 		index += increment;
 
-		*dest++ += vnf->lvolsel * sample;
+		*dest++ += lvolsel * sample;
 	}
 	return index;
 }
@@ -272,13 +330,15 @@ static SLONGLONG MixMonoNormal(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG
 static SLONGLONG MixStereoNormal(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG increment,SLONG todo)
 {
 	SWORD sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
 
 	while(todo--) {
 		sample=srce[index >> FRACBITS];
 		index += increment;
 
-		*dest++ += vnf->lvolsel * sample;
-		*dest++ += vnf->rvolsel * sample;
+		*dest++ += lvolsel * sample;
+		*dest++ += rvolsel * sample;
 	}
 	return index;
 }
@@ -286,17 +346,24 @@ static SLONGLONG MixStereoNormal(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLO
 static SLONGLONG MixSurroundNormal(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG increment,SLONG todo)
 {
 	SWORD sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
 
-	while(todo--) {
-		sample = srce[index >> FRACBITS];
-		index += increment;
+	if(vnf->lvolsel>=vnf->rvolsel) {
+		while(todo--) {
+			sample = srce[index >> FRACBITS];
+			index += increment;
 
-		if(vnf->lvolsel>=vnf->rvolsel) {
-			*dest++ += vnf->lvolsel*sample;
-			*dest++ -= vnf->lvolsel*sample;
-		} else {
-			*dest++ -= vnf->rvolsel*sample;
-			*dest++ += vnf->rvolsel*sample;
+			*dest++ += lvolsel*sample;
+			*dest++ -= lvolsel*sample;
+		}
+	} else {
+		while(todo--) {
+			sample = srce[index >> FRACBITS];
+			index += increment;
+
+			*dest++ -= rvolsel*sample;
+			*dest++ += rvolsel*sample;
 		}
 	}
 	return index;
@@ -305,6 +372,26 @@ static SLONGLONG MixSurroundNormal(SWORD* srce,SLONG* dest,SLONGLONG index,SLONG
 static SLONGLONG MixMonoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG increment,SLONG todo)
 {
 	SLONG sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rampvol = vnf->rampvol;
+
+	if (rampvol) {
+		SLONG oldlvol = vnf->oldlvol - lvolsel;
+		while(todo--) {
+			sample=(SLONG)srce[index>>FRACBITS]+
+			       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+			        *(index&FRACMASK)>>FRACBITS);
+			index += increment;
+
+			*dest++ += ((lvolsel << CLICK_SHIFT) + oldlvol * rampvol)
+					   * sample >> CLICK_SHIFT;
+			if (!--rampvol)
+				break;
+		}
+		vnf->rampvol = rampvol;
+		if (todo < 0)
+			return index;
+	}
 
 	while(todo--) {
 		sample=(SLONG)srce[index>>FRACBITS]+
@@ -312,13 +399,7 @@ static SLONGLONG MixMonoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG
 		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
-		if(vnf->rampvol) {
-			*dest++ += (((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
-			            (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
-			           )*sample>>CLICK_SHIFT;
-			vnf->rampvol--;
-		} else
-			*dest++ += vnf->lvolsel * sample;
+		*dest++ += lvolsel * sample;
 	}
 	return index;
 }
@@ -326,6 +407,30 @@ static SLONGLONG MixMonoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG
 static SLONGLONG MixStereoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG increment,SLONG todo)
 {
 	SLONG sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
+	SLONG rampvol = vnf->rampvol;
+
+	if (rampvol) {
+		SLONG oldlvol = vnf->oldlvol - lvolsel;
+		SLONG oldrvol = vnf->oldrvol - rvolsel;
+		while(todo--) {
+			sample=(SLONG)srce[index>>FRACBITS]+
+			       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+			        *(index&FRACMASK)>>FRACBITS);
+			index += increment;
+
+			*dest++ +=((lvolsel << CLICK_SHIFT) + oldlvol * rampvol)
+					   * sample >> CLICK_SHIFT;
+			*dest++ +=((rvolsel << CLICK_SHIFT) + oldrvol * rampvol)
+					   * sample >> CLICK_SHIFT;
+			if (!--rampvol)
+				break;
+		}
+		vnf->rampvol = rampvol;
+		if (todo < 0)
+			return index;
+	}
 
 	while(todo--) {
 		sample=(SLONG)srce[index>>FRACBITS]+
@@ -333,18 +438,8 @@ static SLONGLONG MixStereoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLO
 		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
-		if(vnf->rampvol) {
-			*dest++ +=(((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
-			           (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
-			          )*sample>>CLICK_SHIFT;
-			*dest++ +=(((SLONG)vnf->oldrvol<<CLICK_SHIFT)+
-			           (SLONG)(vnf->oldrvol-vnf->rvolsel)*vnf->rampvol
-			          )*sample>>CLICK_SHIFT;
-			vnf->rampvol--;
-		} else {
-			*dest++ += vnf->lvolsel * sample;
-			*dest++ += vnf->rvolsel * sample;
-		}
+		*dest++ += lvolsel * sample;
+		*dest++ += rvolsel * sample;
 	}
 	return index;
 }
@@ -352,32 +447,47 @@ static SLONGLONG MixStereoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLO
 static SLONGLONG MixSurroundInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG increment,SLONG todo)
 {
 	SLONG sample;
+	SLONG lvolsel = vnf->lvolsel;
+	SLONG rvolsel = vnf->rvolsel;
+	SLONG rampvol = vnf->rampvol;
+	SLONG oldvol, vol;
+
+	if (lvolsel >= rvolsel) {
+		vol = lvolsel;
+		oldvol = vnf->oldlvol;
+	} else {
+		vol = rvolsel;
+		oldvol = vnf->oldrvol;
+	}
+
+	if (rampvol) {
+		oldvol -= vol;
+		while(todo--) {
+			sample=(SLONG)srce[index>>FRACBITS]+
+			       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+			        *(index&FRACMASK)>>FRACBITS);
+			index += increment;
+
+			sample=((vol << CLICK_SHIFT) + oldvol * rampvol)
+				   * sample >> CLICK_SHIFT;
+			*dest++ += sample;
+			*dest++ -= sample;
+			if (!--rampvol)
+				break;
+		}
+		vnf->rampvol = rampvol;
+		if (todo < 0)
+			return index;
+	}
 
 	while(todo--) {
-		int oldvol,vol;
-
 		sample=(SLONG)srce[index>>FRACBITS]+
 		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
 		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
-		if(vnf->lvolsel>=vnf->rvolsel) {
-			vol=vnf->lvolsel;oldvol=vnf->oldlvol;
-		} else {
-			vol=vnf->rvolsel;oldvol=vnf->oldrvol;
-		}
-
-		if(vnf->rampvol) {
-			sample=(((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
-			        (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
-			       )*sample>>CLICK_SHIFT;
-			*dest++ += sample;
-			*dest++ -= sample;
-			vnf->rampvol--;
-		} else {
-			*dest++ += vol*sample;
-			*dest++ -= vol*sample;
-		}
+		*dest++ += vol*sample;
+		*dest++ -= vol*sample;
 	}
 	return index;
 }
@@ -549,7 +659,8 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 			}
 		} else {
 			/* The sample is playing forward */
-			if((vnf->flags & SF_LOOP) && (vnf->current >= idxlend)) {
+			if((vnf->flags & SF_LOOP) &&
+			   (vnf->current >= idxlend)) {
 				/* the sample is looping, check the loopend index */
 				if(vnf->flags & SF_BIDI) {
 					/* sample is doing bidirectional loops, so 'bounce' the
@@ -596,7 +707,7 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 			if((vnf->current<0x7fffffff)&&(endpos<0x7fffffff)) {
 				if((md_mode & DMODE_INTERP)) {
 					if(vc_mode & DMODE_STEREO) {
-						if((vnf->pan==PAN_SURROUND)&&(vc_mode&DMODE_SURROUND))
+						if((vnf->pan==PAN_SURROUND)&&(md_mode&DMODE_SURROUND))
 							vnf->current=Mix32SurroundInterp
 							           (s,ptr,vnf->current,vnf->increment,done);
 						else
@@ -606,7 +717,7 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 						vnf->current=Mix32MonoInterp
 						               (s,ptr,vnf->current,vnf->increment,done);
 				} else if(vc_mode & DMODE_STEREO) {
-					if((vnf->pan==PAN_SURROUND)&&(vc_mode&DMODE_SURROUND))
+					if((vnf->pan==PAN_SURROUND)&&(md_mode&DMODE_SURROUND))
 						vnf->current=Mix32SurroundNormal
 						               (s,ptr,vnf->current,vnf->increment,done);
 					else
@@ -620,7 +731,7 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 			       {
 				if((md_mode & DMODE_INTERP)) {
 					if(vc_mode & DMODE_STEREO) {
-						if((vnf->pan==PAN_SURROUND)&&(vc_mode&DMODE_SURROUND))
+						if((vnf->pan==PAN_SURROUND)&&(md_mode&DMODE_SURROUND))
 							vnf->current=MixSurroundInterp
 							           (s,ptr,vnf->current,vnf->increment,done);
 						else
@@ -630,7 +741,7 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 						vnf->current=MixMonoInterp
 						               (s,ptr,vnf->current,vnf->increment,done);
 				} else if(vc_mode & DMODE_STEREO) {
-					if((vnf->pan==PAN_SURROUND)&&(vc_mode&DMODE_SURROUND))
+					if((vnf->pan==PAN_SURROUND)&&(md_mode&DMODE_SURROUND))
 						vnf->current=MixSurroundNormal
 						               (s,ptr,vnf->current,vnf->increment,done);
 					else
@@ -690,7 +801,7 @@ void VC1_WriteSamples(SBYTE* buf,ULONG todo)
 					if(vnf->flags&SF_REVERSE) vnf->increment=-vnf->increment;
 					vol = vnf->vol;  pan = vnf->pan;
 
-					vnf->oldlvol=vnf->rvolsel;vnf->oldrvol=vnf->rvolsel;
+					vnf->oldlvol=vnf->lvolsel;vnf->oldrvol=vnf->rvolsel;
 					if(vc_mode & DMODE_STEREO) {
 						if(pan != PAN_SURROUND) {
 							vnf->lvolsel=(vol*(PAN_RIGHT-pan))>>8;
